part of xpx_chain_sdk.api;

class AccountRoutesApi {
  AccountRoutesApi([_ApiClient? _apiClient])
      : _apiClient = _apiClient ?? defaultApiClient;

  final _ApiClient _apiClient;

  static const getAccountInfoRoute = '/account/{accountId}';
  static const getAccountsInfoRoute = '/account';
  static const getAccountMultisigRoute = '/account/{accountId}/multisig';
  static const getAccountMultisigGraphRoute =
      '/account/{accountId}/multisig/graph';
  static const transactionsRoute = '/account/{publicKey}/transactions';
  static const incomingTransactionsRoute =
      '/account/{publicKey}/transactions/incoming';
  static const outgoingTransactionsRoute =
      '/account/{publicKey}/transactions/outgoing';
  static const unconfirmedTransactionsRoute =
      '/account/{publicKey}/transactions/unconfirmed';
  static const aggregateBondedTransactionsRoute =
      '/account/{publicKey}/transactions/partial';
  static const getAccountsNamesRoute = '/account/names';
  static const getAccountPropertiesRoute = '/account/{address}/properties';
  static const getAccountsPropertiesRoute = '/account/properties';

  /// Get account information
  ///
  /// Returns the account information.
  Future<AccountInfo?> getAccountInfo(Address address) async {
    // create path and map variables
    final String path = getAccountInfoRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{accountId}', address.address);

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient.deserialize(response.data, 'AccountInfoDTO');
      return AccountInfo.fromDTO(resp);
    } else {
      return null;
    }
  }

  /// Get accounts information
  ///
  /// Returns the account information for an List of accounts.
  Future<List<AccountInfo>?> getAccountsInfo(List<Address> addresses) async {
    final Object postBody = Addresses.fromList(addresses);

    // create path and map variables
    final String path = getAccountsInfoRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.post(path, postBody);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient
          .deserialize(response.data, 'List<AccountInfoDTO>')
          .cast<AccountInfoDTO>();
      return AccountInfo.listFromDTO(resp);
    } else {
      return [];
    }
  }

  /// Get multisig account information
  ///
  /// Returns the [MultisigAccountInfo] information.
  Future<MultisigAccountInfo?> getAccountMultisig(Address address) async {
    // create path and map variables
    final String path = getAccountMultisigRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{accountId}', address.address);

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      return _apiClient.deserialize(response.data, 'MultisigAccountInfo');
    } else {
      return null;
    }
  }

  /// Get multisig account graph information
  ///
  /// Returns list [MultisigAccountGraphInfo] graph.
  Future<List<MultisigAccountGraphInfo?>> getAccountMultisigGraph(
      Address address) async {
    // create path and map variables
    final String path = getAccountMultisigGraphRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{accountId}', address.address);

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      return _apiClient
          .deserialize(response.data, 'List<MultisigAccountGraphInfo>')
          .cast<MultisigAccountGraphInfo>();
    } else {
      return [];
    }
  }

  /// Get confirmed transactions
  ///
  /// Gets an List of transactions for which an account is the sender or receiver.
  Future<List<Transaction?>> transactions(PublicAccount account,
      {int? pageSize, String? id, String? ordering}) async {
    // create path and map variables
    final String path = transactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{publicKey}', account.publicKey.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }
    if (ordering != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'ordering', ordering));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get incoming transactions
  ///
  /// Gets an List of incoming transactions.
  /// A transaction is said to be incoming with respect to an
  /// account if the account is the recipient of the transaction.
  Future<List<Transaction?>> incomingTransactions(PublicAccount account,
      {int? pageSize, String? id, String? ordering}) async {
    // create path and map variables
    final String path = incomingTransactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{publicKey}', account.publicKey.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }
    if (ordering != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'ordering', ordering));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get outgoing transactions
  ///
  /// Gets an List of outgoing transactions.
  /// A transaction is said to be outgoing with respect to an
  /// account if the account is the sender of the transaction.
  Future<List<Transaction?>> outgoingTransactions(PublicAccount account,
      {int? pageSize, String? id, String? ordering}) async {
    // create path and map variables
    final String path = outgoingTransactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{publicKey}', account.publicKey.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }
    if (ordering != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'ordering', ordering));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get unconfirmed transactions
  ///
  /// Gets the List of transactions not included in a block where an account
  /// is the sender or receiver.
  Future<List<Transaction?>?> unconfirmedTransactions(PublicAccount account,
      {int? pageSize, String? id, String? ordering}) async {
    // create path and map variables
    final String path = unconfirmedTransactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{publicKey}', account.publicKey.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }
    if (ordering != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'ordering', ordering));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get aggregate bonded transactions information
  ///
  /// Gets an List of [aggregate bonded transactions] where the account is
  /// the sender or requires to cosign the transaction.
  Future<List<Transaction?>> partialTransactions(PublicAccount account,
      {int? pageSize, String? id, String? ordering}) async {
    // create path and map variables
    final String path = aggregateBondedTransactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{publicKey}', account.publicKey.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }
    if (ordering != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'ordering', ordering));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get readable names for a set of accountIds.
  Future<List<AccountNames?>> getAccountsNames(List<Address> addresses,
      {int? pageSize, String? id, String? ordering}) async {
    final Object postBody = Addresses.fromList(addresses);

    // verify required params are set
    if (addresses.isEmpty) {
      throw ApiException(400, 'Missing required param: addresses');
    }

    // create path and map variables
    final String path = getAccountsNamesRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.post(path, postBody);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient
          .deserialize(response.data, 'List<AccountNames>')
          .cast<AccountNamesDTO>();
      return AccountNames.listFromJson(resp);
    } else {
      return [];
    }
  }

  Future<AccountProperties?> getAccountProperties(Address address) async {
    // create path and map variables
    final String path = getAccountPropertiesRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{address}', address.address);

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp =
          _apiClient.deserialize(response.data, 'AccountPropertiesDTO');

      return AccountProperties.fromDto(resp);
    } else {
      return null;
    }
  }

  Future<List<AccountProperties>?> getAccountsProperties(
      List<Address> addresses) async {
    final Object postBody = Addresses.fromList(addresses);

    // verify required params are set
    if (addresses.isEmpty) {
      throw ApiException(400, 'Missing required param: addresses');
    }

    // create path and map variables
    final String path =
        getAccountsPropertiesRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.post(path, postBody);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient
          .deserialize(response.data, 'List<AccountPropertiesDTO>')
          .cast<AccountPropertiesDTO>();
      return AccountProperties.listFromJson(resp);
    } else {
      return null;
    }
  }
}
