part of xpx_chain_sdk.api;

class BlockchainRoutesApi {
  BlockchainRoutesApi([_ApiClient? apiClient])
      : _apiClient = apiClient ?? defaultApiClient;

  final _ApiClient _apiClient;

  static const getBlockchainHeightRoute = '/chain/height';
  static const getBlockByHeightRoute = '/block/{height}';
  static const getBlockchainScoreRoute = '/chain/score';
  static const getDiagnosticStorageRoute = '/diagnostic/storage';
  static const getBlockTransactionsRoute = '/block/{height}/transactions';
  static const getBlocksByHeightWithLimitRoute =
      '/blocks/{height}/limit/{limit}';

  /// Get block information
  ///
  /// Gets a [BlockInfo] from the chain that has the given height.
  Future<BlockInfo?> getBlockByHeight(int height) async {
    // create path and map variables
    final String path = getBlockByHeightRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{height}', height.toString());

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient.deserialize(response.data, 'BlockInfoDTO');
      return BlockInfo.fromDTO(resp);
    } else {
      return null;
    }
  }

  /// Get the current height of the chain
  ///
  /// Returns the current height of the blockchain.
  Future<Height> getBlockchainHeight() async {
    // create path and map variables
    final String path = getBlockchainHeightRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient.deserialize(response.data, 'HeightDTO');
      return Height.fromDTO(resp);
    } else {
      return Height(Uint64.zero);
    }
  }

  /// Get the current score of the chain
  ///
  /// Gets the current [BlockchainScore].
  /// The higher the score, the better the chain.
  /// During synchronization, nodes try to get the best blockchain
  /// in the network.
  /// The score for a block is derived from its difficulty and
  /// the time (in seconds)
  /// that has elapsed since the last block:
  /// block score &#x3D; difficulty − time elasped since last block
  Future<BlockchainScore?> getBlockchainScore() async {
    // create path and map variables
    final String path = getBlockchainScoreRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient.deserialize(response.data, 'BlockchainScoreDTO');
      return BlockchainScore.fromDTO(resp);
    } else {
      return null;
    }
  }

  /// Get the storage information
  ///
  /// Returns a [BlockchainStorageInfo] statistical information about
  /// the blockchain.
  Future<BlockchainStorageInfo?> getDiagnosticStorage() async {
    // create path and map variables
    final String path =
        getDiagnosticStorageRoute.replaceAll('{format}', 'json');

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      return _apiClient.deserialize(response.data, 'BlockchainStorageInfo');
    } else {
      return null;
    }
  }

  /// Get transactions from a block
  ///
  /// Returns an List of [Transaction] included in a block for a given
  /// block height.
  Future<List<Transaction?>> getBlockTransactions(int height,
      {int? pageSize, String? id}) async {
    // create path and map variables
    final String path = getBlockTransactionsRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{height}', height.toString());

    // query params
    final List<QueryParam> queryParams = [];
    if (pageSize != null) {
      queryParams.addAll(
          _convertParametersForCollectionFormat('', 'pageSize', pageSize));
    }
    if (id != null) {
      queryParams.addAll(_convertParametersForCollectionFormat('', 'id', id));
    }

    final response = await _apiClient.get(path, queryParams);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final List resp =
          _apiClient.deserialize(response.data, 'List<Transaction>');
      return resp.map(deserializeDTO).toList();
    } else {
      return [];
    }
  }

  /// Get blocks information
  ///
  /// Gets up to limit number of blocks after given block height.
  Future<List<BlockInfo>?> getBlocksByHeightWithLimit(
      int height, int limit) async {
    // create path and map variables
    final String path = getBlocksByHeightWithLimitRoute
        .replaceAll('{format}', 'json')
        .replaceAll('{height}', height.toString())
        .replaceAll('{limit}', limit.toString());

    final response = await _apiClient.get(path);

    if (response.statusCode! >= 400) {
      throw ApiException(response.statusCode!, response.data);
    } else if (response.data != null) {
      final resp = _apiClient
          .deserialize(response.data, 'List<BlockInfoDTO>')
          .cast<BlockInfoDTO>();
      return BlockInfo.listFromDTO(resp);
    } else {
      return [];
    }
  }
}
