part of xpx_chain_sdk.account;

class Account {
  Account._(this.publicAccount, this.account);

  /// Create an Account from a given hex private key.
  static Future<Account> fromPrivateKey(String shex, int networkType) async {
    final account = await crypto.KeyPair.fromPrivateKey(shex);

    final publicKey = ByteUtils.bytesToHex(account.publicKey.toList());

    final publicAccount = PublicAccount.fromPublicKey(publicKey, networkType);
    return Account._(publicAccount, account);
  }

  /// Create an Account from a given networkType.
  static Future<Account> random(int networkType) async {
    final kp = await crypto.KeyPair.random();
    final acc = await Account.fromPrivateKey(
        ByteUtils.bytesToHex(kp.secretKey), networkType);
    final publicAccount = acc.publicAccount;
    final account = acc.account;
    return Account._(publicAccount, account);
  }

  /// Creates an encrypted message from this account to the [recipientPublicAccount].
  Future<EncryptedMessage> encryptMessage(String plainTextMessage,
          PublicAccount recipientPublicAccount) async =>
      EncryptedMessage.create(
          plainTextMessage,
          ByteUtils.bytesToHex(account.secretKey),
          recipientPublicAccount.publicKey,
          HexUtils.isHex(plainTextMessage));

  /// Decrypts an encrypted message received by this account from [senderPublicAccount].
  Future<PlainMessage> decryptMessage(
          EncryptedMessage encryptedMessage, PublicAccount senderPublicAccount,
          [isHexMessage = false]) async =>
      EncryptedMessage.decrypt(
          encryptedMessage,
          ByteUtils.bytesToHex(account.secretKey),
          senderPublicAccount.publicKey,
          isHexMessage);

  PublicAccount publicAccount;
  crypto.KeyPair account;

  String get publicKey => publicAccount.publicKey;

  Address get address => publicAccount.address;

  @override
  String toString() => publicAccount.toString();

  Map<String, dynamic> toJson() =>
      {'publicAccount': publicAccount, 'account': account};

  /// Signs raw data.
  Future<String> signData(String rawData) async {
    final String hex = HexUtils.utf8ToHex(rawData);
    final Uint8List data = HexUtils.hexToBytes(hex);
    final signedData = await account.sign(data);
    return ByteUtils.bytesToHex(signedData.bytes);
  }

  Future<SignedTransaction> signTransaction(
          Transaction tx, String generationHash) async =>
      signTransactionWith(tx, this, generationHash);

  Future<SignedTransaction> signWithCosignatures(Transaction tx,
          List<Account> cosignatories, String generationHash) async =>
      signTransactionWithCosignatures(tx, this, cosignatories, generationHash);

  Future<CosignatureSignedTransaction> signCosignatureTransaction(
          CosignatureTransaction tx) async =>
      signCosignatureTransactionRwa(tx, this);
}

class PublicAccount {
  PublicAccount._(this.publicKey, this.address);

  /// Create an Account from a given publicKey hex string.
  static PublicAccount fromPublicKey(String? publicKey, int networkType) {
    if (publicKey == null ||
        (publicKeySize != publicKey.length && 66 != publicKey.length)) {
      throw errInvalidPublicKey;
    }
    final address = Address.fromPublicKey(publicKey, networkType);
    return PublicAccount._(publicKey, address);
  }

  String publicKey;
  Address address;

  @override
  String toString() => encoder.convert(this);

  /// Verifies a signature.
  Future<bool> verifySignature(String data, String signature) async {
    if (signature.isEmpty) {
      throw errNullSignature;
    }
    if (64 != (signature.length / 2)) {
      throw errInvalidSignature;
    }
    if (signature.length % 2 != 0) {
      throw errInvalidHexadecimal;
    }

    final _publicKey = crypto.SimplePublicKey(hex.decode(publicKey),
        type: crypto.KeyPairType.ed25519);

    final String hexData = HexUtils.utf8ToHex(data);

    final crypto.Signature _signature =
        crypto.Signature(hex.decode(signature), publicKey: _publicKey);

    return crypto.KeyPair.verify(
        data: Uint8List.fromList(hex.decode(hexData)),
        publicKey: Uint8List.fromList(_publicKey.bytes),
        signature: _signature);
  }

  Map<String, dynamic> toJson() => {'address': address, 'publicKey': publicKey};
}

class AccountNames {
  AccountNames._(this.address, this.names);

  static AccountNames fromDto(AccountNamesDTO dto) {
    final address = Address.fromEncoded(dto.address!);
    final names = (dto._names == null) ? null : dto._names!.cast<String>();
    return AccountNames._(address, names);
  }

  /* The address of the account in hexadecimal. */
  Address address;

  /* The mosaic linked namespace names. */
  List<String>? names;

  static List<AccountNames> listFromJson(List<AccountNamesDTO>? json) =>
      json == null ? <AccountNames>[] : json.map(AccountNames.fromDto).toList();

  @override
  String toString() => encoder.convert(this);

  Map<String, dynamic> toJson() => {'address': address, 'names': names};
}

class AccountInfo {
  AccountInfo.fromDTO(AccountInfoDTO v) {
    final List<Mosaic?> mList =
        List.filled(v._account!._mosaics!.length, null, growable: false);
    for (var i = 0; i < v._account!._mosaics!.length; i++) {
      mList[i] = Mosaic.fromDTO(v._account!._mosaics![i]);
    }

    address = Address.fromEncoded(v._account!.address!);
    addressHeight = v._account!.addressHeight!.toUint64();
    publicKey = v._account!._publicKey;
    publicKeyHeight = v._account!._publicKeyHeight!.toUint64();
    accountType = v._account!._accountType;
    linkedAccountKey = v._account!._linkedAccountKey;
    mosaics = mList;
  }

  Address? address;
  Uint64? addressHeight;
  String? publicKey;
  Uint64? publicKeyHeight;
  List<Mosaic?>? mosaics;
  int? accountType;
  String? linkedAccountKey;
  num? reputation;

  @override
  String toString() => encoder.convert(this);

  Map<String, dynamic> toJson() => {
        'address': address,
        'addressHeight': addressHeight,
        'publicKey': publicKey,
        'publicKeyHeight': publicKeyHeight,
        'accountType': accountType,
        'linkedAccountKey': linkedAccountKey,
        'mosaics': mosaics,
        'reputation': reputation
      };

  static List<AccountInfo>? listFromDTO(List<AccountInfoDTO>? dto) =>
      dto == null
          ? null
          : dto.map((value) => AccountInfo.fromDTO(value)).toList();
}
